<?php

namespace OrganizationsManagement\Service;

use OrganizationsManagement\Util\InstanceProvider;
use OrganizationsManagement\Util\Messages;
use OrganizationsManagement\Util\Utilities;
use Teknisa\Model\Teknisa\Entities\OmComandoEstrutura;
use Teknisa\Model\Teknisa\Entities\OmComandoEstruturaUrl;
use Teknisa\Model\Util\NovoCodigo as NovoCodigo;

class Commands {
    /**
     * @param $filters
     * @return array
     */
    public function findCommands($filters) {
        $conditions = Utilities::createQueryFiltered($filters);
        $orgId      = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $structure  = InstanceProvider::getRepository()->getOmComandoEstrutura()->findCommands($orgId, $conditions);
        return $structure;
    }

    /**
     * @param $id
     * @param $dscomandoest
     * @param $nrordem
     * @param $comando
     * @param $idtpcomando
     * @param $idambcomando
     * @param $idacaoexecom
     * @param $idativo
     * @return array
     * @throws \Doctrine\ORM\ORMException
     */
    public function saveCommand($id, $dscomandoest, $nrordem, $comando, $idtpcomando,
                                $idambcomando, $idacaoexecom, $idativo) {
        /** @var OmComandoEstrutura $command */
        $command = InstanceProvider::getRepository()->getOmComandoEstrutura()->findOneBy(
            array('nrordem' => $nrordem, 'idacaoexecom' => $idacaoexecom, 'idambcomando' => $idambcomando));
        if($command && $command->getId() != $id) {
            throw new \Exception(Messages::ERR_SAME_ACTION_ORDER_COMMAND);
        }

        /** @var OmComandoEstrutura $commandEntity */
        $commandEntity = InstanceProvider::getRepository()->getOmComandoEstrutura()->findOneBy(array('id' => $id));
        $commandEntity = InstanceProvider::getFactory()->getComandoEstrutura()->save($commandEntity, $dscomandoest,
            $nrordem, $comando, $idtpcomando, $idambcomando, $idacaoexecom, $idativo);
        InstanceProvider::getEntityManager()->persist($commandEntity);
        InstanceProvider::getEntityManager()->flush();
        return array('ID' => intval($commandEntity->getId()));
    }

    /**
     * @param $id
     * @throws \Doctrine\DBAL\ConnectionException
     * @throws \Exception
     */
    public function removeCommand($id) {
        $entityManager = InstanceProvider::getEntityManager();
        $connection    = $entityManager->getConnection();

        try {
            $connection->beginTransaction();
            $this->removeCommandAndUrl($id);
            $connection->commit();
        } catch (\Exception $ex){
            $connection->rollBack();
            throw $ex;
        }
    }

    /**
     * @param $id
     * @throws \Exception
     */
    private function removeCommandAndUrl($id) {
        $entityManager = InstanceProvider::getEntityManager();

        /** @var OmComandoEstruturaUrl[] $commandUrls */
        $commandUrls = InstanceProvider::getRepository()->getOmComandoEstruturaUrl()->findBy(array('comandoEstruturaId' => $id));
        foreach ($commandUrls as $commandUrl) {
            $entityManager->remove($commandUrl);
        }
        $entityManager->flush();

        /** @var OmComandoEstrutura $commandEntity */
        $commandEntity = InstanceProvider::getRepository()->getOmComandoEstrutura()->findOneBy(array('id' => $id));
        $entityManager->remove($commandEntity);
        $entityManager->flush();
    }

    /**
     * @param $commandId
     * @param $filters
     * @return array
     */
    public function findCommandUrls($commandId, $filters) {
        $conditions = Utilities::createQueryFiltered($filters);
        $orgId      = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $structure  = InstanceProvider::getRepository()->getOmComandoEstrutura()->findCommandUrls($orgId, $commandId, $conditions);
        return $structure;
    }

    /**
     * @param $id
     * @param $commandId
     * @param $url
     * @param $idativo
     * @return array
     * @throws \Doctrine\ORM\ORMException
     */
    public function saveCommandUrl($id, $commandId, $url, $idativo) {
        /** @var OmComandoEstruturaUrl $commandUrl */
        $commandUrl = InstanceProvider::getRepository()->getOmComandoEstruturaUrl()->findOneBy(array('id' => $id));
        $commandUrl = InstanceProvider::getFactory()->getComandoEstrutura()->saveUrl($commandUrl, $commandId, $url, $idativo);
        InstanceProvider::getEntityManager()->persist($commandUrl);
        InstanceProvider::getEntityManager()->flush();
        return array('ID' => intval($commandUrl->getId()));
    }

    /**
     * @param $id
     * @throws \Exception
     */
    public function removeCommandUrl($id) {
        /** @var OmComandoEstruturaUrl $commandUrl */
        $commandUrl = InstanceProvider::getRepository()->getOmComandoEstruturaUrl()->findOneBy(array('id' => $id));
        InstanceProvider::getEntityManager()->remove($commandUrl);
        InstanceProvider::getEntityManager()->flush();
    }

    /**
     * @param $commands
     * @param $url
     * @throws \Doctrine\DBAL\ConnectionException
     * @throws \Exception
     */
    public function saveMultipleCommandUrl($commands, $url)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection    = $entityManager->getConnection();

        try {
            $connection->beginTransaction();
            $urls = explode(';', $url);

            foreach ($commands as $commandId) {
                foreach ($urls as $commandUrl) {
                    if(!empty(trim($commandUrl))) {
                        $commandUrlEntity = InstanceProvider::getFactory()->getComandoEstrutura()->saveUrl(null, $commandId, trim($commandUrl));
                        $entityManager->persist($commandUrlEntity);
                    }
                }
            }

            $entityManager->flush();
            $connection->commit();
        } catch (\Exception $ex){
            $connection->rollBack();
            throw $ex;
        }
    }

    /**
     * @param $origin
     * @param $destiny
     * @throws \Doctrine\DBAL\ConnectionException
     * @throws \Exception
     */
    public function copyCommands($origin, $destiny) {
        $entityManager = InstanceProvider::getEntityManager();
        $connection    = $entityManager->getConnection();

        try {
            $connection->beginTransaction();

            /** @var OmComandoEstrutura[] $oldCommands */
            $oldCommands = InstanceProvider::getRepository()->getOmComandoEstrutura()->findBy(array('idambcomando' => $destiny));
            foreach ($oldCommands as $oldCommand) {
                $this->removeCommandAndUrl($oldCommand->getId());
            }

            /** @var OmComandoEstrutura[] $commands */
            $commands = InstanceProvider::getRepository()->getOmComandoEstrutura()->findBy(array('idambcomando' => $origin, 'idativo' => 'S'));

            foreach ($commands as $command) {
                /** @var OmComandoEstruturaUrl[] $commandUrls */
                $commandUrls = InstanceProvider::getRepository()->getOmComandoEstruturaUrl()->findBy(array('comandoEstruturaId' => $command->getId()));

                $newCommand = clone $command;
                $commandId  = NovoCodigo::geraCodigo(InstanceProvider::getEnvironment()->getCurrentOrganizationId(), "OM_COMANDO_ESTRUTURA", 1, 10, false, false);
                $newCommand->setId($commandId);
                $newCommand->setIdambcomando($destiny);
                $this->setEntityAudit($newCommand);
                $entityManager->persist($newCommand);
                $entityManager->flush();

                foreach ($commandUrls as $commandUrl) {
                    $newCommandUrl = clone $commandUrl;
                    $newCommandUrl->setId(NovoCodigo::geraCodigo(InstanceProvider::getEnvironment()->getCurrentOrganizationId(), "OM_COMANDO_ESTRUTURA_URL", 1, 10, false, false));
                    $newCommandUrl->setComandoEstruturaId($commandId);
                    $this->setEntityAudit($newCommandUrl);
                    $entityManager->persist($newCommandUrl);
                }
            }

            $entityManager->flush();
            $connection->commit();
        } catch (\Exception $ex){
            $connection->rollBack();
            throw $ex;
        }
    }

    private function setEntityAudit($entity) {
        $dateNow     = new \DateTime();
        $nrorg       = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $codOperador = InstanceProvider::getEnvironment()->getCurrentUserId();

        $entity->setCdoperinclusao($codOperador);
        $entity->setDtinclusao($dateNow);
        $entity->setNrorginclusao($nrorg);

        $entity->setCdoperultatu($codOperador);
        $entity->setDtultatu($dateNow);
        $entity->setNrorgultatu($nrorg);
    }
}